// file: <src/lwip_hooks.c>
#include "lwip_hooks.h"
extern void log_packet(uint16_t, u16_t);

const char *get_protocol(u16_t type)
{
    switch (type)
    {
    case 1:
        return "ICMP";
    case 6:
        return "TCP";
    case 17:
        return "UDP";
    default:
        return "-";
    }
}

int lwip_hook_ip4_input(struct pbuf *pbuf, struct netif *input_netif)
{
    const struct ip_hdr *iphdr;
    const struct tcp_hdr *tcphdr;
    const struct udp_hdr *udphdr;
    char ip_address[IP_HLEN];
    u16_t protocol;
    uint16_t port;

    iphdr = (struct ip_hdr *)pbuf->payload;
    u16_t ilen = IPH_HL(iphdr);
    protocol = (u16_t)IPH_PROTO(iphdr);
    if (protocol == 6) {
      //tcphdr = (struct tcp_hdr *)pbuf->payload;
      tcphdr = (struct tcp_hdr *)&((u32_t*)iphdr)[ilen];
      port = lwip_ntohs(tcphdr->dest);
    } else if (protocol == 17) {
      udphdr = (struct udp_hdr *)&((u32_t*)iphdr)[ilen];
      port = lwip_ntohs(udphdr->dest);
    } else {
      port = 0;
    }

    sprintf(ip_address, "%d.%d.%d.%d", ip4_addr1_16_val(iphdr->src), ip4_addr2_16_val(iphdr->src), ip4_addr3_16_val(iphdr->src), ip4_addr4_16_val(iphdr->src));
    ESP_LOGI("HOOK", "%s: %s %u",
             get_protocol((u16_t)IPH_PROTO(iphdr)), ip_address, port);

    log_packet(port, (u16_t)IPH_PROTO(iphdr));

    return ESP_OK;
}
